@@ -5,7 +5,7 @@
  * This library was written at the Laboratory of DNA Information Analysis,
  * Human Genome Center, Institute of Medical Science, University of Tokyo,
  * 4-6-1 Shirokanedai, Minato-ku, Tokyo 108-8639, Japan.
- * Contact: mdehoon@ims.u-tokyo.ac.jp
+ * Contact: mdehoon@c2b2.columbia.edu
  * 
  * Permission to use, copy, modify, and distribute this software and its
  * documentation with or without modifications and for any purpose and
@@ -27,9 +27,6 @@
  * 
  */
 
-#ifndef C_CLUSTERING_LIB
-#define C_CLUSTERING_LIB
-
 #ifndef CALL 
 # define CALL
 #endif
@@ -96,4 +93,3 @@ double CALL median (int n, double x[]);
 
 double* calculate_weights(int nrows, int ncolumns, double** data, int** mask,
   double weights[], int transpose, char dist, double cutoff, double exponent);
-#endif
